# miodi

Dependency Injection tools for Dart.

This library aims to easily manage dependencies in your business logic components.

The project is in beta testing, please write about all the errors found.

https://pub.dartlang.org/packages/miodi

## Docs

Documentation coming soon, please wait.

### Example

Added example project. Pls check for future updates for more information. 
https://github.com/Miolin/miodi/tree/master/example

Create component:

```dart

class AppComponent extends MiodiComponent {
  static String testScope = 'test';
  final String param;

  AppComponent(this.param);

  @override
  List<MiodiModule> get modules => [
        module([
          singleton(Database, () => DatabaseImpl()),
          factory(User, () => User())
        ]),
        module([
          singleton(Logger, () => FileLogger(param)),
          factory(LogSender, () => LogSender(get())),
          scope(testScope, ScopeObject, () => ScopeObject())
        ])
      ];
}

class ScopeObject {}

abstract class Database {}

class DatabaseImpl implements Database {}

class User {}

abstract class Logger {}

class FileLogger implements Logger {
  final String tag;
  FileLogger(this.tag);
}

class LogSender {
  final Logger logger;
  LogSender(this.logger);
}

```


In the body of the component, use the ```get()``` method to get dependencies (from the current or other modules).
You can use several components.
You can also use the component's constructor to represent dependencies in modules.

Usage example:

```dart
final appComponent = AppComponent('TAG');

void main() async {
  Miodi.init([appComponent]);

  Database database = inject();
  //if you have a lot of big components override method 'String get componentNamespace' in your components
  //this will speed up the search for dependencies
  User user = inject('customComponentNamespace');
  
  
  ScopeObject scopeObject = scopeInject(AppComponent.testScope);
  
  //For cleaning of scopes it is NECESSARY!! use cleaning methods.
  //otherwise they will work as singleton
  clearScope(AppComponent.testScope);
  
  //and use 'injectAsync' if object creation takes time or requires delays
  Logger fileLogger = await injectAsync();

  LogSender sender = inject();
  sender.sendLog();
}

```


Please be careful when using scopeInject. Scopes in status progress. 

If you have ideas on how to implement self-cleaning scopes , please contact me for prompt implementation.