import 'dart:collection';

import 'consts.dart';
import 'module.dart';
import 'providers.dart';
import '../error/errors.dart';

abstract class MiodiComponent {
  final HashMap<Type, Provider> componentConfiguration = HashMap();
  final HashMap<String, Map<Type, ScopeProvider>> scopeConfiguration = HashMap();
  MiodiComponent() {
    modules.forEach((module) {
      componentConfiguration.addAll(module.moduleConfiguration);
    });
    modules.forEach((module) {
      scopeConfiguration.addAll(module.scopeConfiguration);
    });
  }

  List<MiodiModule> get modules;

  String get componentNamespace => defaultNamespace;

  T get<T>() {
    if (componentConfiguration.containsKey(T))
      return componentConfiguration[T].provide() as T;

    T obj;

    scopeConfiguration.forEach((scope, config) {
      if (obj == null)
        if (config.containsKey(T))
          obj = config[T].provide();
    });

    if (obj == null) throw NotFoundProviderException(T);
    return obj;
  }
}