class FactoryProvider<T> extends Provider<T> {
  FactoryProvider(Type T, Creator<T> creator): super(T, creator);

  @override
  T provide() => creator();
}

class SingletonProvider<T> extends Provider<T> {
  T _instance;
  SingletonProvider(Type T, Creator<T> creator) : super(T, creator);

  @override
  T provide() {
    if (_instance == null) _instance = creator();
    return _instance;
  }
}

class DefaultScopeProvider<T> extends ScopeProvider<T> {
  T _instance;
  DefaultScopeProvider(Type T, String scope, Creator<T> creator): super(T, scope,creator);

  @override
  T provide() {
    if (_instance == null) _instance = creator();
    return _instance;
  }

  @override
  void clear() {
    _instance = null;
  }
}

abstract class ScopeProvider<T> extends Provider<T> {
  String _scope;
  ScopeProvider(Type T, this._scope, creator) : super(T, creator);
  String get scope => _scope;

  void clear();
}

abstract class Provider<T> {
  final Type _type;
  final Creator<T> _creator;
  Provider(this._type, this._creator);
  Type get provideType => _type;
  Creator<T> get creator => _creator;

  T provide();
}

typedef T Creator<T>();