library miodi;

import 'dart:async';
import 'core/component.dart';
import 'core/configuration.dart';
import 'core/module.dart';
import 'core/providers.dart';

export 'core/component.dart';
export 'core/consts.dart';
export 'core/module.dart';

T inject<T>([String componentNamespace]) => Miodi._instance._configuration.provide(componentNamespace);

Future<T> injectAsync<T>([String componentNamespace]) =>
    Miodi._instance._configuration.provideAsync(componentNamespace);

T scopeInject<T>(String scope, [String componentNamespace]) => Miodi._instance._configuration.scopeConfiguration.provide(scope, componentNamespace);

Future<T> scopeInjectAsync<T>(String scope, [String componentNamespace]) =>
    Miodi._instance._configuration.scopeConfiguration.provideAsync(scope, componentNamespace);

Provider<T> singleton<T>(Type T, Creator<T> creator) =>
    SingletonProvider(T, creator);

Provider<T> factory<T>(Type T, Creator<T> creator) => FactoryProvider(T, creator);

Provider<T> scope<T>(String scope, Type T, Creator<T> creator) => DefaultScopeProvider(T, scope, creator);

MiodiModule module(List<Provider> providers) => MiodiModule(providers);

class Miodi {
  final MiodiConfiguration _configuration;

  Miodi._(this._configuration);

  static Miodi _instance;
  static void init(List<MiodiComponent> components) {
    if (_instance != null) return;
    _instance = Miodi._(MiodiConfiguration(components));
  }
}

//region Clear scopes functions

void clearScope(String scope, [String componentNamespace]) {
  Miodi._instance._configuration.scopeConfiguration.clearScope(scope, componentNamespace);
}

void clearAllScope([String componentNamespace]) {
  Miodi._instance._configuration.scopeConfiguration.clearAllScope(componentNamespace);
}

void clearScopes(Iterable<String> scopes) {
  Miodi._instance._configuration.scopeConfiguration.clearScopes(scopes);
}

void clearInactiveScopes(Iterable<String> activeScopes) {
  Miodi._instance._configuration.scopeConfiguration.clearInactiveScopes(activeScopes);
}

void clearScopesWithNamespace(Map<String, Iterable<String>> inactiveScopes) {
  Miodi._instance._configuration.scopeConfiguration.clearScopesWithNamespace(inactiveScopes);
}

void clearInactiveScopesWithNamespace(Map<String, Iterable<String>> activeScopes) {
  Miodi._instance._configuration.scopeConfiguration.clearInactiveScopesWithNamespace(activeScopes);
}

Future clearScopesAsync(Iterable<String> scopes) async {
  clearScopes(scopes);
  return;
}

Future clearScopesWithNamespaceAsync(Map<String, Iterable<String>> inactiveScopes) async {
  clearScopesWithNamespace(inactiveScopes);
  return;
}

Future clearInactiveScopesWithNamespaceAsync(Map<String, Iterable<String>> activeScopes) async {
  clearInactiveScopesWithNamespace(activeScopes);
  return;
}

Future clearInactiveScopesAsync(Iterable<String> activeScopes) async {
  clearInactiveScopes(activeScopes);
  return;
}

Future clearAllScopeAsync([String componentNamespace]) async {
  clearAllScope(componentNamespace);
  return;
}

Future clearScopeAsync(String scope, [String componentNamespace]) async {
  clearScope(scope, componentNamespace);
  return;
}
//endregion
